(function(ns) {
    var _earth = null,
        _dataProcess = null,
        _seTrackMgr = null,

        PASS_ATTR_MAP = {
            'flyHeight': undefined,
            'speed': '速度'
        },
        INTERVAL = 10,
        _data = {},
        _tracks = null,
        _dynamics = null,
        _dynamicTree = null,
        _trails = null,
        _posIndex = null,
        _timer = null,
        _mainDynamic = null,
        _mainTrack = null,
        _trackType = {};


    //main
    function _initGps(earth, dataProcess) {
        _earth = earth;
        _dataProcess = dataProcess

        _seTrackMgr = new ns.STAMP.SeTrackManager({
            earth: _earth,
            dataProcess: _dataProcess
        });


        _initUI();
        _addEventListeners();

        _getDynamic();
    }

    function _destroyGps() {
        _removeEventListeners();
        ns.clearInterval(_timer);
        if (_tracks) {
            for (var i in _tracks) {
                if (_tracks[i] && _tracks[i]['newid']) {
                    _earth.DynamicSystem.UnLoadDynamicObject(_tracks[i]['newid']);
                }
            }
        }
    }

    function _initUI() {

    }

    function _addEventListeners() {

    }

    function _removeEventListeners() {

    }

    function _map(k) {
        if (!k) {
            return undefined;
        }

        return PASS_ATTR_MAP[k];
    }

    function _attr(o, k, v) {
        try {
            if (!o) {
                return undefined;
            }
            var ks = k.split('.');
            var ot = o;
            var i = 0;
            for (i = 0; i < ks.length - 1; i++) {
                if (!ot[ks[i]]) {
                    ot[ks[i]] = {};
                }
                ot = ot[ks[i]];
            }
            if (typeof v != 'undefined') {
                ot[ks[i]] = v;
            }
            return ot[ks[i]];
        } catch (e) {

        }

        return undefined;
    }

    function _valueOrDefault(obj, property, defaultValue) {
        try {
            if (!obj) {
                return defaultValue;
            }
            var val = _attr(obj, property);
            if (typeof val == 'undefined') {
                return defaultValue;
            }
            return val;
        } catch (e) {
            return defaultValue;
        }
    }


    /**
     * _dynamics:{
     *     id:'id,positionId,trackId',
     *     dynamic:{ISEDynamicObject}
     * }
     **/


    function _getDynamic() {
        _earth.Event.OnDynamicListLoaded = function(list) {
            _earth.Event.OnDynamicListLoaded = function() {};

            if (list == null || list.Count == 0) {
                return;
            }

            if (_dynamics) {
                _dynamics.splice(0, _dynamics.length);
            } else {
                _dynamics = [];
            }
            for (var i = 0; i < list.Count; i++) {
                var dynamic = list.Items(i);
                var type = dynamic.Type;
                try {
                    /*
                    if ((type == "DynamicObject") || (type == "DynamicPeople")) {
                        _dynamics.push(dynamic);
                    }
                    */
                    if (type == "DynamicObject") {
                        for (var j = 0; j < 4; j++) {
                            _dynamics.push({
                                id: dynamic.Guid + '_' + j,
                                dynamic: dynamic
                            });
                        }
                    }
                    if (type == "DynamicPeople") {
                        for (var j = 0; j < 3; j++) {
                            _dynamics.push({
                                id: dynamic.Guid + '_' + j,
                                dynamic: dynamic
                            });
                        }
                    }
                } catch (e) {
                    continue;
                }
            }

            _createTree();
            _getPosition();
        };
        _earth.DynamicSystem.ApplyDynamicList();
    }

    function _createTree() {
        var treeData = [{
            id: -1,
            name: '车',
            open: true,
            isParent: true,
            trackType: 'rootcar'
        }, {
            id: -2,
            name: '人',
            open: true,
            isParent: true,
            trackType: 'rootpeople'
        }];
        if (_dynamics) {
            for (var i = 0; i < _dynamics.length; i++) {
                var node = {
                    id: _dynamics[i].id,
                    name: _dynamics[i]['dynamic'].Name + _dynamics[i].id.split('_')[1],
                    isParent: false
                };
                if (_dynamics[i]['dynamic'].Type == 'DynamicObject') {
                    node.pId = -1;
                } else if (_dynamics[i]['dynamic'].Type == 'DynamicPeople') {
                    node.pId = -2;
                } else {
                    continue;
                }
                treeData.push(node);
            }
        }

        var setting = {
            data: {
                simpleData: {
                    enable: true
                },
                keep: {
                    parent: true
                }
            },
            edit: {
                enable: true,
                showRemoveBtn: false,
                showRenameBtn: false,
                drag: {
                    prev: false,
                    inner: false,
                    next: false,
                    isMove: false,
                    isCopy: false
                }
            },
            view: {
                autoCancelSelected: false,
                selectedMulti: false,
                showTitle: false,
                addDiyDom: function(treeId, treeNode) {
                    if (treeNode.level != 1) {
                        return;
                    }

                    var tid = treeNode.tId;
                    var dd = '<span id="' + tid + '_diy_button" class="cs-dynamic cs-dynamic-lock" ' +
                        'style="margin-top:2px;" ' +
                        'data-type="3" ' +
                        'onclick="STAMP.onDiyButtonClick(event,\'' + treeId + '\',\'' + tid + '\')"></span>';
                    $('#gpsPanel>#gpsContainer').find("#gpsTree").find('#' + tid + '_a').after(dd);
                }
            },
            callback: {
                onClick: function(event, treeId, node) {
                    if (!node) {
                        return;
                    }

                    _onTreeNodeClick(event, treeId, node);
                }
            }
        };

        _dynamicTree = $.fn.zTree.init($('#gpsPanel>#gpsContainer').find("#gpsTree"), setting, treeData);
    }

    function _getPosition() {
        var listFile = _earth.Environment.RootPath + '\\trail\\trailList.xml';
        var tlist = _earth.UserDocument.LoadXmlFile(listFile);
        tlist = ns.STAMP.seUtil.xml2json(tlist);
        try {
            if (tlist.record) {
                if (!(tlist.record instanceof Array)) {
                    tlist.record = [tlist.record];
                }
                _trails = {};
                for (var i = 0; i < tlist.record.length; i++) {
                    var trail = tlist.record[i].id;
                    var type = tlist.record[i].type;
                    trail = _earth.Environment.RootPath + '\\trail\\trackList' + trail + '.xml';
                    trail = _earth.UserDocument.LoadXmlFile(trail);
                    trail = ns.STAMP.seUtil.xml2json(trail);

                    if (trail && trail.Route && trail.Route.Pass) {
                        var ps = trail.Route.Pass;
                        if (!(ps instanceof Array)) {
                            ps = [ps];
                        }
                        _trails[type] = ps;
                    }
                }

                _posIndex = {};
                var ci = 0,
                    pi = 0;
                for (var i = 0; i < _dynamics.length; i++) {
                    if (_dynamics[i]['dynamic'].Type == 'DynamicObject') {
                        _posIndex[_dynamics[i].id] = ci * 5;
                        ci++;
                    } else if (_dynamics[i]['dynamic'].Type == 'DynamicPeople') {
                        _posIndex[_dynamics[i].id] = pi * 5;
                        pi++;
                    } else {
                        continue;
                    }
                }

                _getPositions();
                _timer = ns.setInterval(function() {
                    _getPositions();
                }, INTERVAL * 1000);
            }
        } catch (e) {

        }
    }

    function _getPositions(id) {
        function _getPos(id) {
            for (var i = 0; i < _dynamics.length; i++) {
                if (_dynamics[i].id == id) {
                    var pos = {};
                    var p = null;
                    if (_dynamics[i]['dynamic'].Type == 'DynamicObject') {
                        p = _trails['car'][_posIndex[_dynamics[i].id]];
                    } else if (_dynamics[i]['dynamic'].Type == 'DynamicPeople') {
                        p = _trails['people'][_posIndex[_dynamics[i].id]];
                    } else {
                        continue;
                    }
                    if (!p) {
                        continue;
                    }
                    pos['position'] = {
                        x: p.Longitude,
                        y: p.Latitude,
                        z: p.Altitude
                    };
                    pos['id'] = _dynamics[i]['dynamic'].Guid;
                    var now = new Date();
                    var y = now.getFullYear();
                    var m = now.getMonth() + 1;
                    m = m > 9 ? m : '0' + m;
                    var d = now.getDate();
                    d = d > 9 ? d : '0' + d;
                    var h = now.getHours();
                    h = h > 9 ? h : '0' + h;
                    var mi = now.getMinutes();
                    mi = mi > 9 ? mi : '0' + mi;
                    var s = now.getSeconds();
                    s = s > 9 ? s : '0' + s;
                    now = y + '-' + m + '-' + d + ' ' + h + ':' + mi + ':' + s;
                    pos['attr'] = {
                        time: now,
                        flyHeight: 0,
                        speed: 1.5
                    }

                    _posIndex[_dynamics[i].id] ++;
                    return pos;
                }
            }
        }

        var positions = {};
        var ids = null;
        if (id) {
            ids = id.split(',');
        } else {
            ids = [];
            for (var i = 0; i < _dynamics.length; i++) {
                ids.push(_dynamics[i].id);
            }
        }
        for (var i = 0; i < ids.length; i++) {
            var o = _getPos(ids[i]);
            if (o) {
                positions[ids[i]] = o;
            }
            // break;
        }

        _onGetPositions(positions);
        // return positions;
    }

    function _onGetPositions(positions) {
        // _updateTrack(positions);
        // _updatePose();

        _updateTrack2(positions);
        _updatePose2();
    }

    /**
     * positions:{
     *     'id':{
     *        position:{
     *            x:0,
     *            y:0,
     *            z:0,
     *        },
     *        id:'dynamicId',
     *        attr:{
     *            time:'2014-01-02',
     *            flyHeight:0,
     *            speed:5,
     *            ...
     *        }
     *     },
     *     ...
     * }
     **/

    /**
     * _tracks:{
     *     'id':{
     *         track:ISETrack,
     *         id:'dynamicid',
     *         attr:{
     *             'passid':{
     *                 '速度':3,
     *                 ...
     *              },
     *             ...
     *         }
     *     }
     * }
     **/



    function _updateTrack(positions) {
        if (!positions) {
            return;
        }
        if (!_tracks) {
            _tracks = {};
        }
        for (var i in positions) {
            if (!_tracks[i]) {
                _tracks[i] = {};
            }
            var pos = positions[i];
            var track = _tracks[i];
            try {
                if (pos['position']) {
                    if (!track['track']) {
                        var g = _earth.Factory.CreateGuid();
                        track['track'] = _earth.Factory.CreateTrack(g, 'track');
                        g = _earth.Factory.CreateGuid();
                        var route = _earth.Factory.CreateStationRoute(g, 'route');
                        route.Yaw = 0;
                        route.Pitch = 0;
                        route.Rate = 0.01;
                        track['track'].AddStation(route);
                    }
                    if (pos['id']) {
                        track['id'] = pos['id'];
                    }
                    var g = _earth.Factory.CreateGuid();
                    if (!track['attr']) {
                        track['attr'] = {};
                    }
                    if (pos['attr']) {
                        track['attr'][g] = {};
                        for (var j in pos['attr']) {
                            var k = _map(j);
                            if (typeof k == 'undefined') {
                                continue;
                            }
                            _attr(track['attr'][g], k, pos['attr'][k]);
                        }
                    }
                    var route = track['track'].GetChildAt(0);
                    var count = route.GetChildCount();
                    var time = _valueOrDefault(pos, 'attr.time', '');
                    var name = (count + 1) + (time ? ':' + time : '');
                    name = time;
                    if (isNaN(pos['position'].x) || isNaN(pos['position'].y) || isNaN(pos['position'].z)) {
                        continue;
                    }
                    var speed = _valueOrDefault(pos['attr'], 'speed', 0);
                    if (speed == 0 && count > 0) {
                        var p2 = route.GetChildAt(count - 1);
                        var v3s = _earth.Factory.CreateVector3s();
                        v3s.Add(p2.Longitude, p2.Latitude, p2.Altitude);
                        v3s.Add(pos['position'].x, pos['position'].y, pos['position'].z);
                        speed = v3s.Length / INTERVAL;
                        // speed = Math.floor(speed);
                        p2.Speed = speed;
                    }
                    var pass = _earth.Factory.CreateStationPass(g, name);
                    pass.Longitude = pos['position'].x;
                    pass.Latitude = pos['position'].y;
                    pass.Altitude = pos['position'].z;
                    pass.FlyHeight = _valueOrDefault(pos['attr'], 'flyHeight', 0);
                    pass.Speed = speed;
                    route.AddStation(pass);
                    track['track'].CommitChanges();
                }
            } catch (e) {
                continue;
            }
        }
    }

    function _updatePose() {
        if (!_tracks) {
            return;
        }
        for (var i in _tracks) {
            try {
                var track = _tracks[i]['track'];
                if (track) {
                    var route = track.GetChildAt(0);
                    if (route) {
                        var n = route.GetChildCount();
                        if (n > 1) {
                            var did = _tracks[i]['id'];
                            if (did) {
                                if (!_tracks[i]['load']) {
                                    (function(j) {
                                        _earth.Event.OnDocumentChanged = function(type, newGuid) {
                                            _earth.Event.OnDocumentChanged = function() {};

                                            if (type != 2) {
                                                return;
                                            }
                                            _tracks[j]['load'] = true;
                                            _tracks[j]['newid'] = newGuid;
                                            _playTrack(track, newGuid);
                                        }
                                        _earth.DynamicSystem.LoadDynamicObject(did);
                                    })(i);
                                } else {
                                    _playTrack(track, _tracks[i]['newid']);
                                }
                            }
                        }
                    }
                }

            } catch (e) {
                continue;
            }
        }
    }

    function _playTrack(track, did) {
        if (!track || !did) {
            return;
        }
        _earth.Event.OnTrackFinish = function(tId, dId) {

            _earth.GlobeObserver.StopTracking();
            _earth.GlobeObserver.Stop();

            // _playTrack(track,did);
            var t = _earth.TrackControl.GetTrack(tId);
            _delPlayedPass(t);
        }

        switch (track.Status) {
            case 0:
                track.BindObject = did;
                _earth.TrackControl.SetMainTrack(track.Guid, 3);
                track.UpdateRate(2);
                track.CommitChanges();
                track.Play(false);
                break;
            case 1:
                break;
            case 2:
                track.Resume();
                break;
        }
    }

    function _delPlayedPass(track) {
        if (!track) {
            return;
        }
        if (track.GetChildCount() > 0) {
            var route = track.GetChildAt(0);
            /*
            for (var i = 0, n = route.GetChildCount(); i < n - 1; i++) {
                var pass = route.GetChildAt(i);
                track.DeleteStation(pass.Guid);
            }
            */
            while (route.GetChildCount() > 1) {
                var pass = route.GetChildAt(0);
                track.DeleteStation(pass.Guid);
            }
            track.CommitChanges();
        }
    }



    /**
     * _tracks:{
     *     'id':{
     *         tracks:[{
     *             track:ISETrack,
     *             played:false
     *         },
     *         ...
     *         ],
     *         id:'dynamicid',
     *         attrs:{
     *             'trackid':{
     *                 'passid':{
     *                     '速度':3,
     *                     ...
     *                 },
     *                 ...
     *             },
     *             ...
     *         }
     *     }
     * }
     **/

    function _calcSpeed(p1, p2, interval) {
        try {
            var v3s = _earth.Factory.CreateVector3s();
            v3s.Add(p1.Longitude, p1.Latitude, p1.Altitude);
            v3s.Add(p2.Longitude, p2.Latitude, p2.Altitude);
            speed = v3s.Length / interval;
            speed = Math.ceil(speed);
            return speed;
        } catch (e) {
            return 0;
        }
    }

    function _setPass(pos, dtid, dpid, stid, spid) {
        try {
            var id = pos['id'];
            if (!_tracks || !dtid || !dpid) {
                return;
            }
            if (!_tracks[id]) {
                _tracks[id] = {};
            }
            var pns = 'attrs.' + dtid + '.' + dpid + '.';
            var dataSource = pos['attr'];
            var map = true;
            if (stid && spid) {
                dataSource = _attr(_tracks[id], 'attrs.' + stid + '.' + spid);
                //复制已有pass的attr,不必再做映射
                map = false;
            }
            if (dataSource) {
                for (var j in dataSource) {
                    var k = j;
                    if (map) {
                        k = _map(j);
                    }
                    if (typeof k == 'undefined') {
                        continue;
                    }
                    _attr(_tracks[id], pns + k, dataSource[j]);
                }
            }
        } catch (e) {

        }
    }

    function _updateTrack2(positions) {
        if (!positions) {
            return;
        }
        if (!_tracks) {
            _tracks = {};
        }
        for (var i in positions) {
            if (!_tracks[i]) {
                _tracks[i] = {};
            }
            var pos = positions[i];
            var trackData = _tracks[i];
            try {
                if (pos['position']) {
                    if (!trackData['tracks']) {
                        trackData['tracks'] = [];
                    }
                    var track = null;
                    var tl = trackData['tracks'].length;
                    if (tl > 0 && trackData['tracks'][tl - 1]['track'].Status == 0 && !trackData['tracks'][tl - 1]['played']) {
                        track = trackData['tracks'][tl - 1]['track'];
                    } else {
                        var g = _earth.Factory.CreateGuid();
                        track = _earth.Factory.CreateTrack(g, 'track');
                        track.Visibility = false;
                        g = _earth.Factory.CreateGuid();
                        var route = _earth.Factory.CreateStationRoute(g, 'route');
                        route.Yaw = 0;
                        route.Pitch = 0;
                        route.Rate = 0.01;
                        track.AddStation(route);

                        if (tl > 0) {
                            var trackTemp = trackData['tracks'][tl - 1]['track'];
                            try {
                                var routeTemp = trackTemp.GetChildAt(0);
                                var pn = routeTemp.GetChildCount();
                                var passTemp = routeTemp.GetChildAt(pn - 1);
                                g = _earth.Factory.CreateGuid();
                                var pass = _earth.Factory.CreateStationPass(g, passTemp.Name);
                                pass.Longitude = passTemp.Longitude;
                                pass.Latitude = passTemp.Latitude;
                                pass.Altitude = passTemp.Altitude;
                                pass.FlyHeight = passTemp.FlyHeight;
                                pass.Speed = passTemp.Speed;
                                route.AddStation(pass);
                                _setPass(pos, track.Guid, g, trackTemp.Guid, passTemp.Guid);
                            } catch (e) {
                                continue;
                            }
                        }

                        trackData['tracks'].push({
                            track: track,
                            played: false
                        });
                    }

                    if (isNaN(pos['position'].x) || isNaN(pos['position'].y) || isNaN(pos['position'].z)) {
                        continue;
                    }

                    if (pos['id']) {
                        trackData['id'] = pos['id'];
                    }
                    var g = _earth.Factory.CreateGuid();
                    var time = _valueOrDefault(pos, 'attr.time', '');
                    var name = time;
                    var speed = _valueOrDefault(pos['attr'], 'speed', 0);
                    var route = track.GetChildAt(0);
                    var pass = _earth.Factory.CreateStationPass(g, name);
                    pass.Longitude = pos['position'].x;
                    pass.Latitude = pos['position'].y;
                    pass.Altitude = pos['position'].z;
                    pass.FlyHeight = _valueOrDefault(pos['attr'], 'flyHeight', 0);
                    var count = route.GetChildCount();
                    if (speed == 0 && count > 0) {
                        speed = _calcSpeed(route.GetChildAt(count - 1), pass, INTERVAL);
                    }
                    pass.Speed = speed;
                    route.AddStation(pass);
                    track.CommitChanges();

                    _setPass(pos, track.Guid, g);
                }
            } catch (e) {
                continue;
            }
        }
    }

    function _updatePose2() {
        if (!_tracks) {
            return;
        }

        var loadList = [];
        for (var i in _tracks) {
            try {
                var tracks = _tracks[i]['tracks'];
                if (!tracks || tracks.length == 0) {
                    continue;
                }
                var track = null;
                for (var j = 0; j < tracks.length; j++) {
                    if (!tracks[j]['played']) {
                        track = tracks[j]['track'];
                        break;
                    }
                }
                if (track) {
                    var route = track.GetChildAt(0);
                    var pn = route.GetChildCount();
                    if (pn > 1) {
                        function _play(track, j) {
                            if (_tracks[j]['load']) {
                                _playTrack2(track, _tracks[j]['newid']);
                            } else {
                                ns.setTimeout(_play(track, j), 100);
                            }
                        }
                        _play(track, i);
                    } else if (pn == 1) {
                        if (!_tracks[i]['load']) {
                            var did = _tracks[i]['id'];
                            /*
                            (function(j) {
                                var k = j;
                                _earth.Event.OnDocumentChanged = function(type, newGuid) {
                                    // _earth.Event.OnDocumentChanged = function() {};

                                    if (type != 2) {
                                        return;
                                    }
                                    _tracks[k]['load'] = true;
                                    _tracks[k]['newid'] = newGuid;

                                    var dynamicObj = _earth.DynamicSystem.GetSphericalObject(newGuid);
                                    var st = track.GetPose(0);
                                    dynamicObj.SphericalTransform.SetPose(st.Longitude, st.Latitude, st.Altitude, st.Heading, 0, 0);
                                }
                                _earth.DynamicSystem.LoadDynamicObject(did);
                            })(i);
                            */

                            loadList.push({
                                id: i,
                                did: did
                            });
                        }
                    }
                }
            } catch (e) {
                continue;
            }
        }

        if (loadList.length > 0) {
            var i = 0;
            _earth.Event.OnDocumentChanged = function(type, newGuid) {
                if (type == 2) {
                    _tracks[loadList[i].id]['load'] = true;
                    _tracks[loadList[i].id]['newid'] = newGuid;
                }
                i++;
                if (i < loadList.length) {
                    _earth.DynamicSystem.LoadDynamicObject(loadList[i].did);
                } else {
                    _earth.Event.OnDocumentChanged = function() {};
                }
            };
            _earth.DynamicSystem.LoadDynamicObject(loadList[i].did);
        }
    }

    function _loadDynamic(did, id) {
        _earth.Event.OnDocumentChanged = function(type, newGuid) {
            // _earth.Event.OnDocumentChanged = function() {};

            if (type != 2) {
                return;
            }
            _tracks[id]['load'] = true;
            _tracks[id]['newid'] = newGuid;

            // var dynamicObj = _earth.DynamicSystem.GetSphericalObject(newGuid);
            // var st = track.GetPose(0);
            // dynamicObj.SphericalTransform.SetPose(st.Longitude, st.Latitude, st.Altitude, st.Heading, 0, 0);
        }
        _earth.DynamicSystem.LoadDynamicObject(did);
    }

    function _playTrack2(track, did) {
        if (!track || !did) {
            return;
        }
        _earth.Event.OnTrackFinish = function(tId, dId) {

            _earth.GlobeObserver.StopTracking();
            _earth.GlobeObserver.Stop();

            var id = null;
            for (var i in _tracks) {
                if (_tracks[i]['newid'] == dId) {
                    // id = _tracks[i]['id'];
                    id = i;
                    break;
                }
            }

            _setTrackPlayed(id, tId);
        }

        switch (track.Status) {
            case 0:
                track.BindObject = did;
                track.InitFollowTrack(180, 15, 4, 0);
                if (_mainDynamic == did) {
                    _earth.TrackControl.SetMainTrack(track.Guid, 3);
                    _mainTrack = track.Guid;
                } else {
                    // _earth.TrackControl.SetMainTrack(track.Guid, 4);
                }
                track.UpdateRate(2);
                track.CommitChanges();
                track.Play(false);
                if (_mainTrack) {
                    var node = _dynamicTree.getSelectedNodes();
                    var tt = _valueOrDefault(_trackType, node[0].id, 3);
                    _earth.TrackControl.SetMainTrack(_mainTrack, tt);
                }
                break;
            case 1:
                break;
            case 2:
                track.Resume();
                break;
        }
    }

    function _setTrackPlayed(did, tid) {
        if (!did || !tid) {
            return;
        }
        var tracks = _tracks[did]['tracks'];
        if (tracks) {
            var i = 0;
            for (i = 0; i < tracks.length; i++) {
                var track = tracks[i]['track'];
                if (track.Guid == tid) {
                    tracks[i]['played'] = true;
                    break;
                } else {
                    _earth.TrackControl.DeleteTrack(track.Guid);
                    tracks.splice(i, 1);
                    i--;
                }
            }
            if (i < tracks.length - 1) {
                _playTrack2(tracks[i + 1]['track'], _tracks[did]['newid']);
            }
        }
    }

    function _onTreeNodeClick(event, treeId, node) {
        if (!node) {
            return;
        }

        if (node.level == 1) {
            if (!_tracks[node.id]) {
                return;
            }

            var tracks = _tracks[node.id]['tracks'];
            if (!tracks || tracks.length == 0) {
                return;
            }
            var i = 0;
            for (i = 0; i < tracks.length; i++) {
                var track = tracks[i]['track'];
                var played = tracks[i]['played'];
                if (track.Status == 1 || !played) {
                    var tt = _valueOrDefault(_trackType, node.id, 3);
                    _earth.TrackControl.SetMainTrack(track.Guid, tt);
                    _mainDynamic = _tracks[node.id]['newid'];
                    _mainTrack = track.Guid;
                    break;
                }
            }
            if (i == tracks.length) {
                if (_mainTrack) {
                    _earth.TrackControl.SetMainTrack(_mainTrack, 4);
                    _mainTrack = null;
                }
                var track = tracks[i - 1]['track'];
                var st = track.GetPose(1);
                _earth.GlobeObserver.GotoLookat(st.Longitude, st.Latitude, st.Altitude, st.Heading, 15, 0, 10);
                _mainDynamic = _tracks[node.id]['newid'];
            }
        } else if (node.level == 0) {
            if (_mainTrack) {
                _earth.TrackControl.SetMainTrack(_mainTrack, 4);
                _mainTrack = null;
            }
            _mainDynamic = null;
        }
    }

    function _onDiyButtonClick(event, treeId, tid) {
        var node = _dynamicTree.getNodeByTId(tid);
        _dynamicTree.selectNode(node, false);
        _onTreeNodeClick(event, treeId, node);

        var sp = $('#gpsPanel>#gpsContainer').find("#gpsTree").find('#' + tid + '_diy_button');
        if (sp.attr('data-type') == 3) {
            if (_mainTrack) {
                _earth.TrackControl.SetMainTrack(_mainTrack, 2);
                _attr(_trackType, node.id, 2);
            }
            sp.removeClass('cs-dynamic-lock').addClass('cs-dynamic-unlock').attr('data-type', 2);
        } else if (sp.attr('data-type') == 2) {
            if (_mainTrack) {
                _earth.TrackControl.SetMainTrack(_mainTrack, 3);
                _attr(_trackType, node.id, 3);
            }
            sp.removeClass('cs-dynamic-unlock').addClass('cs-dynamic-lock').attr('data-type', 3);
        }
    }





    if (!ns.STAMP.initer) {
        ns.STAMP.initer = {};
    }
    ns.STAMP.initer.initGps = _initGps;
    ns.STAMP.initer.destroyGps = _destroyGps;
    ns.STAMP.onDiyButtonClick = _onDiyButtonClick;
})(window);
